@@ -30,7 +30,7 @@ async fn wrap_main() -> Result<(), ErrorWrap> {
             distribute::template(template).map_err(ErrorWrap::from)
         }
         cli::Arguments::Pull(pull) => distribute::pull(pull).await.map_err(ErrorWrap::from),
-        cli::Arguments::Run(pull) => distribute::run_local(pull).await.map_err(ErrorWrap::from),
+        cli::Arguments::Run(run) => distribute::run_local(run).await.map_err(ErrorWrap::from),
         cli::Arguments::ServerStatus(s) => {
             distribute::server_status(s).await.map_err(ErrorWrap::from)
         }
