@@ -4,14 +4,19 @@ pub mod python;
 pub mod requirements;
 
 #[cfg(feature = "cli")]
-use crate::transport;
+mod hashing;
 
-use derive_more::{Constructor, Display, From, Unwrap};
+#[cfg(feature = "cli")]
+use crate::client::execute::FileMetadata;
+
+use derive_more::{Constructor, Display, From};
 use serde::de::DeserializeOwned;
 use serde::{Deserialize, Serialize};
 use std::path::Path;
 use std::path::PathBuf;
 
+use getset::Getters;
+
 macro_rules! const_port {
     ($NUMERIC:ident, $STR:ident, $value:expr) => {
         pub const $NUMERIC: u16 = $value;
@@ -27,7 +32,7 @@ const_port!(CLIENT_PORT, CLIENT_PORT_STR, 8953);
 const_port!(CLIENT_KEEPALIVE_PORT, CLIENT_KEEPALIVE_PORT_STR, 8954);
 const_port!(CLIENT_CANCEL_PORT, CLIENT_CANCEL_PORT_STR, 8955);
 
-#[derive(Debug, Display, thiserror::Error, From)]
+#[derive(Debug, Display, thiserror::Error, Constructor)]
 #[display(
     fmt = "configuration file: `{}` reason: `{}`",
     "configuration_file",
@@ -40,9 +45,22 @@ pub struct ConfigurationError {
 
 #[derive(Debug, Display, From)]
 pub enum ConfigErrorReason {
-    #[display(fmt = "deserialization error: {}", _0)]
-    Deserialization(serde_yaml::Error),
-    MissingFile(std::io::Error),
+    #[display(fmt = "{}", _0)]
+    Deserialization(DeserError),
+    #[display(fmt = "missing file: {}", _0)]
+    MissingFile(MissingFileNameError),
+    #[display(fmt = "General Io error when opening config file: {}", _0)]
+    IoError(std::io::Error),
+}
+
+#[derive(Debug, Display, Constructor)]
+#[display(
+    fmt = "general deserialization: {general}\npython deserialization err: {python_err}\nappatainer deserialization error: {apptainer_err}"
+)]
+pub struct DeserError {
+    general: serde_yaml::Error,
+    python_err: serde_yaml::Error,
+    apptainer_err: serde_yaml::Error,
 }
 
 #[derive(Debug, From, thiserror::Error)]
@@ -67,7 +85,7 @@ pub struct CanonicalizeError {
     err: std::io::Error,
 }
 
-#[derive(Debug, Display, From, thiserror::Error)]
+#[derive(Debug, Display, From, thiserror::Error, Constructor)]
 #[display(fmt = "Error loading configuration for jobs {:?} ", path)]
 /// happens when a file path does not contain a filename
 pub struct MissingFileNameError {
@@ -138,82 +156,218 @@ impl Node {
     }
 }
 
+#[derive(Deserialize, Serialize, Clone, Debug)]
+#[serde(deny_unknown_fields)]
+/// special type that can be sent over TCP with bincode
+pub enum TransportJobs<FILE> {
+    Python(PythonConfig<FILE>),
+    Apptainer(ApptainerConfig<FILE>),
+}
+
+impl<F> From<Jobs<F>> for TransportJobs<F> {
+    fn from(x: Jobs<F>) -> TransportJobs<F> {
+        match x {
+            Jobs::Python(py) => TransportJobs::Python(py),
+            Jobs::Apptainer(app) => TransportJobs::Apptainer(app),
+        }
+    }
+}
+
 #[derive(Debug, Clone, Deserialize, Serialize, From)]
+#[cfg_attr(test, derive(derive_more::Unwrap))]
 #[serde(untagged)]
 #[serde(deny_unknown_fields)]
-pub enum Jobs {
-    Python(PythonConfig),
-    Apptainer(ApptainerConfig),
+pub enum Jobs<FILE> {
+    Python(PythonConfig<FILE>),
+    Apptainer(ApptainerConfig<FILE>),
 }
 
-#[derive(Debug, Clone, Deserialize, Serialize, Constructor)]
+impl<F> From<TransportJobs<F>> for Jobs<F> {
+    fn from(x: TransportJobs<F>) -> Jobs<F> {
+        match x {
+            TransportJobs::Python(py) => Jobs::Python(py),
+            TransportJobs::Apptainer(app) => Jobs::Apptainer(app),
+        }
+    }
+}
+
+#[derive(
+    Debug, Clone, Deserialize, Serialize, Constructor, getset::Getters, getset::MutGetters,
+)]
 #[serde(deny_unknown_fields)]
-#[cfg_attr(feature = "python", pyo3::pyclass)]
-pub struct ApptainerConfig {
+pub struct ApptainerConfig<FILE> {
+    #[getset(get = "pub(crate)")]
     meta: Meta,
     #[serde(rename = "apptainer")]
-    description: apptainer::Description,
+    #[getset(get = "pub(crate)", get_mut = "pub(crate)")]
+    description: apptainer::Description<FILE>,
 }
 
-#[derive(Debug, Clone, Deserialize, Serialize, Constructor)]
+#[derive(Debug, Clone, Deserialize, Serialize, Constructor, Getters)]
 #[serde(deny_unknown_fields)]
-#[cfg_attr(feature = "python", pyo3::pyclass)]
-pub struct PythonConfig {
+pub struct PythonConfig<FILE> {
+    #[getset(get = "pub(crate)")]
     meta: Meta,
     #[serde(rename = "python")]
-    description: python::Description,
+    #[getset(get = "pub(crate)")]
+    description: python::Description<FILE>,
 }
 
-#[derive(Debug, Clone, Deserialize, Serialize, Constructor)]
+#[derive(Debug, Clone, Deserialize, Serialize, From)]
+pub enum Init {
+    Python(python::Initialize<common::HashedFile>),
+    Apptainer(apptainer::Initialize<common::HashedFile>),
+}
+
+impl From<&Jobs<common::HashedFile>> for Init {
+    fn from(config: &Jobs<common::HashedFile>) -> Init {
+        match &config {
+            Jobs::Apptainer(app) => Init::Apptainer(app.description.initialize.clone()),
+            Jobs::Python(py) => Init::Python(py.description.initialize.clone()),
+        }
+    }
+}
+
+#[cfg(feature = "cli")]
+impl Init {
+    pub(crate) fn sendable_files(&self, is_user: bool) -> Vec<FileMetadata> {
+        let mut out = Vec::new();
+
+        match &self {
+            Self::Python(py) => py.sendable_files(is_user, &mut out),
+            Self::Apptainer(app) => app.sendable_files(is_user, &mut out),
+        }
+
+        out
+    }
+
+    pub(crate) fn delete_files(self) -> Result<(), std::io::Error> {
+        match self {
+            Self::Python(py) => {
+                py.python_build_file_path.delete_at_hashed_path()?;
+                common::delete_hashed_files(py.required_files)
+            }
+            Self::Apptainer(app) => {
+                app.sif.delete_at_hashed_path()?;
+                common::delete_hashed_files(app.required_files)
+            }
+        }
+    }
+}
+
+#[derive(Debug, Clone, Deserialize, Serialize, From)]
+pub enum Job {
+    Python(python::Job<common::HashedFile>),
+    Apptainer(apptainer::Job<common::HashedFile>),
+}
+
+#[cfg(feature = "cli")]
+impl Job {
+    pub(crate) fn sendable_files(&self, is_user: bool) -> Vec<FileMetadata> {
+        let mut out = Vec::new();
+
+        match &self {
+            Self::Python(py) => py.sendable_files(is_user, &mut out),
+            Self::Apptainer(app) => app.sendable_files(is_user, &mut out),
+        }
+
+        out
+    }
+
+    pub(crate) fn name(&self) -> &str {
+        match &self {
+            Self::Python(py) => &py.name(),
+            Self::Apptainer(app) => &app.name(),
+        }
+    }
+
+    pub(crate) fn delete_files(self) -> Result<(), std::io::Error> {
+        match self {
+            Self::Python(py) => {
+                py.python_job_file().delete_at_hashed_path()?;
+                common::delete_hashed_files(py.required_files)
+            }
+            Self::Apptainer(app) => common::delete_hashed_files(app.required_files),
+        }
+    }
+
+    #[cfg(test)]
+    pub(crate) fn placeholder_apptainer() -> Self {
+        let job = apptainer::Job::new("some_job".into(), vec![]);
+        Self::from(job)
+    }
+
+    #[cfg(test)]
+    pub(crate) fn placeholder_python(file: common::File) -> Self {
+        let file = file.hashed().unwrap();
+        let job = python::Job::new("python".into(), file, vec![]);
+        Self::from(job)
+    }
+}
+
+#[derive(Debug, Clone, Deserialize, Serialize, Constructor, Getters)]
 #[serde(deny_unknown_fields)]
-#[cfg_attr(feature = "python", pyo3::pyclass)]
 pub struct Meta {
+    #[getset(get = "pub(crate)")]
     pub batch_name: String,
+    #[getset(get = "pub(crate)")]
     pub namespace: String,
     pub matrix: Option<matrix_notify::OwnedUserId>,
     pub capabilities: requirements::Requirements<requirements::JobRequiredCaps>,
 }
 
 #[cfg(feature = "cli")]
-impl Jobs {
+impl Jobs<common::File> {
     pub fn len_jobs(&self) -> usize {
         match &self {
             Self::Python(pyconfig) => pyconfig.description.len_jobs(),
             Self::Apptainer(apptainer_config) => apptainer_config.description.len_jobs(),
         }
     }
-    pub async fn load_jobs(&self) -> Result<JobOpts, LoadJobsError> {
+
+    /// ensure that all paths exist as we expect them to
+    pub(crate) fn verify_config(&self) -> Result<(), ConfigErrorReason> {
+        match &self {
+            Self::Python(py) => py.description.verify_config()?,
+            Self::Apptainer(app) => app.description.verify_config()?,
+        };
+
+        Ok(())
+    }
+
+    pub fn hashed(&self) -> Result<Jobs<common::HashedFile>, MissingFileNameError> {
         match &self {
             Self::Python(pyconfig) => {
-                let py_jobs = pyconfig.description.load_jobs().await?;
-                Ok(py_jobs.into())
+                let description = pyconfig.description.hashed()?;
+
+                Ok(Jobs::from(PythonConfig {
+                    meta: pyconfig.meta.clone(),
+                    description,
+                }))
             }
             Self::Apptainer(apptainer_config) => {
-                let sin_jobs = apptainer_config.description.load_jobs().await?;
-                Ok(sin_jobs.into())
+                let description = apptainer_config.description.hashed()?;
+
+                Ok(Jobs::from(ApptainerConfig {
+                    meta: apptainer_config.meta.clone(),
+                    description,
+                }))
             }
         }
     }
+}
 
-    pub async fn load_build(&self) -> Result<transport::BuildOpts, LoadJobsError> {
+#[cfg(feature = "cli")]
+impl Jobs<common::HashedFile> {
+    pub(crate) fn sendable_files(&self, is_user: bool) -> Vec<FileMetadata> {
         match &self {
-            Self::Python(pyconfig) => {
-                let py_build = pyconfig
-                    .description
-                    .load_build(pyconfig.meta.batch_name.clone())
-                    .await?;
-                Ok(py_build.into())
-            }
-            Self::Apptainer(apptainer_config) => {
-                let sin_build = apptainer_config
-                    .description
-                    .load_build(apptainer_config.meta.batch_name.clone())
-                    .await?;
-                Ok(sin_build.into())
-            }
+            Jobs::Python(py) => py.description.sendable_files(is_user),
+            Jobs::Apptainer(app) => app.description.sendable_files(is_user),
         }
     }
+}
 
+impl<FILE> Jobs<FILE> {
     pub(crate) fn capabilities(
         &self,
     ) -> &requirements::Requirements<requirements::JobRequiredCaps> {
@@ -246,13 +400,35 @@ impl Jobs {
 
     pub fn job_names(&self) -> Vec<&str> {
         match self {
-            Self::Python(py) => py.description.jobs.iter().map(|job| job.name()).collect(),
-            Self::Apptainer(apt) => apt.description.jobs.iter().map(|job| job.name()).collect(),
+            Self::Python(py) => py
+                .description
+                .jobs
+                .iter()
+                .map(|job| job.name().as_str())
+                .collect(),
+            Self::Apptainer(apt) => apt
+                .description
+                .jobs
+                .iter()
+                .map(|job| job.name().as_str())
+                .collect(),
         }
     }
 }
 
-impl Jobs {
+impl<FILE> Jobs<FILE>
+where
+    FILE: Serialize,
+{
+    /// write the config file to a provided `Write`r
+    pub fn to_writer<W: std::io::Write>(&self, writer: W) -> Result<(), serde_yaml::Error> {
+        serde_yaml::to_writer(writer, &self)?;
+
+        Ok(())
+    }
+}
+
+impl ApptainerConfig<common::File> {
     /// write the config file to a provided `Write`r
     pub fn to_writer<W: std::io::Write>(&self, writer: W) -> Result<(), serde_yaml::Error> {
         serde_yaml::to_writer(writer, &self)?;
@@ -265,7 +441,10 @@ pub trait NormalizePaths {
     fn normalize_paths(&mut self, base: PathBuf);
 }
 
-impl NormalizePaths for Jobs {
+impl<FILE> NormalizePaths for Jobs<FILE>
+where
+    FILE: NormalizePaths,
+{
     fn normalize_paths(&mut self, base: PathBuf) {
         match self {
             Self::Python(py) => py.description.normalize_paths(base),
@@ -280,22 +459,41 @@ impl NormalizePaths for Nodes {
     }
 }
 
-#[derive(derive_more::From, Serialize, Deserialize, Clone, Debug, Unwrap)]
-#[serde(deny_unknown_fields)]
-#[cfg(feature = "cli")]
-pub enum JobOpts {
-    Python(Vec<transport::PythonJob>),
-    Apptainer(Vec<transport::ApptainerJob>),
-}
-
 pub fn load_config<T: DeserializeOwned + NormalizePaths>(
     path: &Path,
 ) -> Result<T, ConfigurationError> {
-    let file = std::fs::File::open(path)
-        .map_err(|e| (path.display().to_string(), ConfigErrorReason::from(e)))?;
+    let file = std::fs::File::open(path).map_err(|e| {
+        ConfigurationError::new(path.display().to_string(), ConfigErrorReason::from(e))
+    })?;
 
-    let mut config: T = serde_yaml::from_reader(file)
-        .map_err(|e| (path.display().to_string(), ConfigErrorReason::from(e)))?;
+    let config: Result<T, _> = serde_yaml::from_reader(file);
+
+    let mut config = match config {
+        Err(e) => {
+            let file = std::fs::File::open(path).map_err(|e| {
+                ConfigurationError::new(path.display().to_string(), ConfigErrorReason::from(e))
+            })?;
+            let python_err =
+                serde_yaml::from_reader::<_, PythonConfig<common::File>>(file).unwrap_err();
+
+            let file = std::fs::File::open(path).map_err(|e| {
+                ConfigurationError::new(path.display().to_string(), ConfigErrorReason::from(e))
+            })?;
+            let apptainer_err =
+                serde_yaml::from_reader::<_, ApptainerConfig<common::File>>(file).unwrap_err();
+
+            let deser_error = DeserError {
+                general: e,
+                python_err,
+                apptainer_err,
+            };
+            return Err(ConfigurationError::new(
+                path.display().to_string(),
+                ConfigErrorReason::from(deser_error),
+            ));
+        }
+        Ok(config) => config,
+    };
 
     config.normalize_paths(path.parent().unwrap().to_owned());
 
@@ -311,43 +509,29 @@ fn serialize_nodes() {
 #[test]
 fn serialize_jobs_python() {
     let bytes = include_str!("../../static/example-jobs-python.yaml");
-    let _out: Jobs = serde_yaml::from_str(bytes).unwrap();
+    let _out: Jobs<common::File> = serde_yaml::from_str(bytes).unwrap();
 }
 
 #[test]
 fn serialize_jobs_apptainer() {
     let bytes = include_str!("../../static/example-jobs-apptainer.yaml");
-    let _out: Jobs = serde_yaml::from_str(bytes).unwrap();
+    let _out: Jobs<common::File> = serde_yaml::from_str(bytes).unwrap();
 }
 
 #[cfg(test)]
 mod tests {
     use super::*;
 
-    #[derive(Deserialize)]
-    #[serde(deny_unknown_fields)]
-    struct PythonConfiguration {
-        meta: Meta,
-        python: python::Description,
-    }
-
-    #[derive(Deserialize)]
-    #[serde(deny_unknown_fields)]
-    struct ApptainerConfiguration {
-        meta: Meta,
-        apptainer: apptainer::Description,
-    }
-
     #[test]
     fn serialize_python() {
         let bytes = include_str!("../../static/example-jobs-python.yaml");
-        let _out: PythonConfiguration = serde_yaml::from_str(bytes).unwrap();
+        let _out: super::PythonConfig<common::File> = serde_yaml::from_str(bytes).unwrap();
     }
 
     #[test]
     fn serialize_apptainer() {
         let bytes = include_str!("../../static/example-jobs-apptainer.yaml");
-        let _out: ApptainerConfiguration = serde_yaml::from_str(bytes).unwrap();
+        let _out: ApptainerConfig<common::File> = serde_yaml::from_str(bytes).unwrap();
     }
 
     #[test]
