@@ -54,9 +54,18 @@ pub use matrix_notify::{OwnedUserId, UserId};
 
 #[cfg(feature = "cli")]
 pub use {
-    add::add, client::client_command, kill::kill, node_status::node_status, pause::pause,
-    pull::pull, run_local::run_local, server::server_command, server::RemainingJobs,
-    server_status::get_current_jobs, server_status::server_status, template::template,
+    add::add,
+    client::client_command,
+    kill::kill,
+    node_status::node_status,
+    pause::pause,
+    pull::pull,
+    run_local::run_local,
+    server::server_command,
+    server::RemainingJobs,
+    server_status::get_current_jobs,
+    server_status::server_status,
+    template::template,
 };
 
 #[cfg(test)]
@@ -102,6 +111,7 @@ impl LoggingOutput {
 
 // helper macro to create the subscriber since each individual `$writer` is a distinct type,
 // and they are difficult / impossible to express as boxed trait objects
+#[cfg(feature = "cli")]
 macro_rules! subscriber_helper {
     ($writer:expr, $with_filename:expr, $level:expr) => {
         let subscriber = tracing_subscriber::FmtSubscriber::builder()
