@@ -4,15 +4,18 @@ use crate::prelude::*;
 use std::sync::atomic::AtomicBool;
 use std::sync::atomic::Ordering;
 
-use super::send_files::{ClientSendFilesState, SendFiles};
+use super::send_files::{self, SendFiles};
+
+type ServerSendFilesState = send_files::ReceiverState<send_files::ReceiverFinalStore>;
+type ClientSendFilesState = send_files::SenderState<send_files::SenderFinalStore>;
 
 #[derive(Default, Debug)]
 pub(crate) struct Executing;
 
 pub(crate) struct ClientExecutingState {
     pub(super) conn: transport::Connection<ClientMsg>,
-    pub(super) working_dir: PathBuf,
-    pub(super) job: transport::JobOpt,
+    pub(super) working_dir: WorkingDir,
+    pub(super) run_info: server::pool_data::RunTaskInfo,
     pub(super) folder_state: client::BindingFolderState,
     pub(super) cancel_addr: SocketAddr,
 }
@@ -20,23 +23,22 @@ pub(crate) struct ClientExecutingState {
 pub(crate) struct ServerExecutingState {
     pub(super) conn: transport::Connection<ServerMsg>,
     pub(super) common: super::Common,
-    pub(super) job_name: String,
     pub(super) save_location: PathBuf,
     // pub(super) here so we can pull this out of the state if we need to
-    pub(super) task_info: server::pool_data::RunTaskInfo,
+    pub(super) run_info: server::pool_data::RunTaskInfo,
 }
 
 impl ServerExecutingState {
     fn job_identifier(&self) -> JobSetIdentifier {
-        self.task_info.identifier
+        self.run_info.identifier
     }
 
     fn namespace(&self) -> &str {
-        &self.task_info.namespace
+        &self.run_info.namespace
     }
 
     fn batch_name(&self) -> &str {
-        &self.task_info.batch_name
+        &self.run_info.batch_name
     }
 }
 
@@ -55,7 +57,7 @@ pub(crate) enum ServerError {
 }
 
 impl Machine<Executing, ClientExecutingState> {
-    #[instrument(skip(self), fields(job_name=self.state.job.name()))]
+    #[instrument(skip(self), fields(job_name=self.state.run_info.task.name()))]
     pub(crate) async fn execute_job(
         mut self,
     ) -> Result<
@@ -67,7 +69,7 @@ impl Machine<Executing, ClientExecutingState> {
         let is_cancelled = Arc::new(AtomicBool::new(false));
 
         let working_dir = self.state.working_dir.clone();
-        let job = self.state.job.clone();
+        let run_info = self.state.run_info.clone();
 
         // start an arbiter to monitor the cancellation port and receive information
         // from the main compute process about our job.
@@ -82,13 +84,13 @@ impl Machine<Executing, ClientExecutingState> {
         //
         // This is required because we cannot simultaneously run the task and cancel the
         // task with the same `tokio::select!` call
-        let msg = match job {
-            transport::JobOpt::Python(python_job) => {
+        let msg = match run_info.task {
+            config::Job::Python(python_job) => {
                 let run_result =
                     client::run_python_job(python_job, &working_dir, &mut rx_cancel).await;
                 ClientMsg::from_run_result(run_result)
             }
-            transport::JobOpt::Apptainer(apptainer_job) => {
+            config::Job::Apptainer(apptainer_job) => {
                 let run_result = client::run_apptainer_job(
                     apptainer_job,
                     &working_dir,
@@ -167,14 +169,14 @@ impl Machine<Executing, ClientExecutingState> {
         Machine::from_state(state)
     }
 
-    async fn into_send_files_state(self) -> super::send_files::ClientSendFilesState {
+    async fn into_send_files_state(self) -> ClientSendFilesState {
         debug!("moving client executing -> send files");
         let ClientExecutingState {
             conn,
             working_dir,
-            job,
             folder_state,
             cancel_addr,
+            run_info,
         } = self.state;
 
         #[allow(unused_mut)]
@@ -183,14 +185,16 @@ impl Machine<Executing, ClientExecutingState> {
         #[cfg(test)]
         assert!(conn.bytes_left().await == 0);
 
-        let job_name = job.name().to_string();
-        super::send_files::ClientSendFilesState {
-            conn,
+        let job_name = run_info.task.name().to_string();
+
+        let extra = send_files::SenderFinalStore {
             working_dir,
             job_name,
             folder_state,
             cancel_addr,
-        }
+        };
+
+        ClientSendFilesState { conn, extra }
     }
 
     async fn into_prepare_build_state(self) -> super::prepare_build::ClientPrepareBuildState {
@@ -220,9 +224,9 @@ impl Machine<Executing, ServerExecutingState> {
         skip(self, scheduler_tx), 
         fields(
             node_meta = %self.state.common.node_meta,
-            namespace = self.state.task_info.namespace,
-            batch_name = self.state.task_info.batch_name,
-            job_name = self.state.job_name,
+            namespace = self.state.run_info.namespace,
+            batch_name = self.state.run_info.batch_name,
+            job_name = self.state.run_info.task.name(),
         )
     )]
     pub(crate) async fn wait_job_execution(
@@ -230,8 +234,10 @@ impl Machine<Executing, ServerExecutingState> {
         // the handler to the job scheduler that we can use
         // to notify if any issues arise
         scheduler_tx: &mut mpsc::Sender<server::JobRequest>,
-    ) -> Result<Either<super::PrepareBuildServer, super::SendFilesServer>, (Self, ServerError)>
-    {
+    ) -> Result<
+        Either<super::PrepareBuildServer, super::SendFilesServer<send_files::ReceiverFinalStore>>,
+        (Self, ServerError),
+    > {
         let mut cancelled = false;
 
         let job_identifier = self.state.job_identifier();
@@ -283,7 +289,7 @@ impl Machine<Executing, ServerExecutingState> {
                 info!(
                     "{} job was cancelled (job name {}, batch name {})",
                     self.state.common.node_meta,
-                    self.state.job_name,
+                    self.state.run_info.task.name(),
                     self.state.batch_name()
                 );
                 let tmp = self.state.conn.transport_data(&ServerMsg::Continue).await;
@@ -297,7 +303,7 @@ impl Machine<Executing, ServerExecutingState> {
 
                 let finish_msg = server::pool_data::FinishJob {
                     ident: self.state.job_identifier(),
-                    job_name: self.state.job_name.clone(),
+                    job_name: self.state.run_info.task.name().to_string(),
                 };
 
                 let mark_finished_msg = server::JobRequest::FinishJob(finish_msg);
@@ -345,7 +351,7 @@ impl Machine<Executing, ServerExecutingState> {
         let ServerExecutingState {
             conn,
             common,
-            task_info,
+            run_info,
             ..
         } = self.state;
         let conn = conn.update_state();
@@ -355,10 +361,10 @@ impl Machine<Executing, ServerExecutingState> {
             state.common.node_meta
         );
 
-        (Machine::from_state(state), task_info)
+        (Machine::from_state(state), run_info)
     }
 
-    async fn into_send_files_state(self) -> super::send_files::ServerSendFilesState {
+    async fn into_send_files_state(self) -> ServerSendFilesState {
         debug!(
             "moving {} server executing -> send files",
             self.state.common.node_meta
@@ -366,9 +372,8 @@ impl Machine<Executing, ServerExecutingState> {
         let ServerExecutingState {
             conn,
             common,
-            job_name,
             save_location,
-            task_info,
+            run_info,
         } = self.state;
 
         #[allow(unused_mut)]
@@ -377,12 +382,12 @@ impl Machine<Executing, ServerExecutingState> {
         #[cfg(test)]
         assert!(conn.bytes_left().await == 0);
 
-        super::send_files::ServerSendFilesState {
+        let extra = send_files::ReceiverFinalStore { common, run_info };
+
+        ServerSendFilesState {
             conn,
-            common,
-            task_info,
-            job_name,
             save_location,
+            extra,
         }
     }
 
@@ -569,16 +574,21 @@ async fn cancel_run() {
     let folder_path = PathBuf::from("./tests/python_sleep/");
     let file_to_execute = folder_path.join("sleep30s.py");
 
-    let file_bytes = include_bytes!("../../tests/python_sleep/sleep30s.py");
+    let exec_file = config::common::File::new("./tests/python_sleep/sleep30s.py").unwrap();
 
-    let work_dir = folder_path.join("run");
+    let job = config::python::Job::new("sleep_job".into(), exec_file.clone(), vec![]);
+
+    let work_dir = WorkingDir::from(folder_path.join("run"));
 
     assert_eq!(folder_path.exists(), true);
     assert_eq!(file_to_execute.exists(), true);
 
     // set up the distribute environment
     // this is a little frail depending on how APIs shift in the future
-    client::utils::clean_output_dir(&work_dir).await.unwrap();
+    work_dir.delete_and_create_folders().await.unwrap();
+
+    // load all the required files into the folders before we run
+    work_dir.copy_job_files_python(&job).await;
 
     let keepalive_port = 10_007;
     let transport_port = 10_008;
@@ -601,20 +611,23 @@ async fn cancel_run() {
     //
     // setup client state
     //
-    let job = transport::JobOpt::from(transport::PythonJob {
-        python_file: file_bytes.to_vec(),
-        job_name: "test_job".into(),
-        job_files: vec![],
-    });
+
+    let job_identifier = server::JobSetIdentifier::Identity(1);
+    let run_info = server::pool_data::RunTaskInfo {
+        namespace: "test_namespace".into(),
+        batch_name: "test_batchname".into(),
+        identifier: job_identifier,
+        task: config::Job::placeholder_python(exec_file),
+    };
 
     let folder_state = client::execute::BindingFolderState::new();
 
     let client_state = ClientExecutingState {
         conn: client_conn,
         working_dir: work_dir.clone(),
-        job,
         folder_state,
         cancel_addr,
+        run_info: run_info.clone(),
     };
 
     //
@@ -623,21 +636,12 @@ async fn cancel_run() {
 
     let (cancel_tx, common) =
         protocol::Common::test_configuration(transport_addr, keepalive_addr, cancel_addr);
-    let job_identifier = server::JobSetIdentifier::Identity(1);
-
-    let task_info = server::pool_data::RunTaskInfo {
-        namespace: "test_namespace".into(),
-        batch_name: "test_batchname".into(),
-        identifier: job_identifier,
-        task: transport::JobOpt::placeholder_test_data(),
-    };
 
     let server_state = ServerExecutingState {
         conn: server_conn,
         common,
-        job_name: "test_name".into(),
-        save_location: work_dir.join("server_backup"),
-        task_info,
+        save_location: work_dir.base().join("server_backup"),
+        run_info: run_info.clone(),
     };
 
     //
@@ -700,5 +704,5 @@ async fn cancel_run() {
 
     // clean up the folder after
     assert_eq!(work_dir.exists(), true, "workdir does not exist somehow");
-    std::fs::remove_dir_all(work_dir).unwrap();
+    std::fs::remove_dir_all(work_dir.base()).unwrap();
 }
