@@ -8,6 +8,8 @@ capabilities = ["apptainer"]
 meta = distribute.metadata(namespace, batch_name, capabilities, matrix_user)
 
 sif_path = "./path/to/some/container.sif"
+sif = distribute.file(sif_path, relative = True)
+
 # the contents of file.h5 will appear in the /input directory under the name `initial_condition.h5`
 initial_condition = distribute.file("./path/to/some/file.h5", relative=True, alias="initial_condition.h5")
 # a list of files that will /always/ be present in the /input directory of the container
@@ -15,7 +17,7 @@ required_files = [initial_condition]
 # a list of paths inside the container that should be mounted to a folder on the host system.
 required_mounts = ["/solver/extra_mount"]
 
-initialize = distribute.initialize(sif_path, required_files, required_mounts)
+initialize = distribute.initialize(sif, required_files, required_mounts)
 
 #
 # then put together two jobs that will be run using the container 
